using System;
using System.IO;
using System.Collections;
using System.Collections.Generic;
using System.Text;
using System.Linq;
using System.Text.RegularExpressions;
using System.Data.SQLite;
//#Include:System.Data.dll
//#Include:System.Data.SQLite.dll

namespace PandoraBox
{
    public class CBVideoUniqPlugin : IPluginMacro
    {
        Regex _ytBlock = new Regex(@"{PUNIQYOUTUBE(?<mp>#\d+)*}(.*?){/PUNIQYOUTUBE}", RegexOptions.Compiled | RegexOptions.Singleline);

        public string Execute(string template, PluginMacroArgs args)
        {
            template = Load(template, _ytBlock, GetIniParameter("Youtube.Directory", args), args);
            return template;
        }

        private string Load(string template, Regex regex, String typePath, PluginMacroArgs args)
        {
            return regex.Replace(template, delegate (Match match)
            {
                //получаем текущий ключ
                String key = CleanKey(GetKey(args, match));
                bool dbMode = GetIniParameter("Main.UseDatabase", args) == "1";

                //получаем список использованных элементов
                List<string> usedVideos = args.Context[typePath + "UniqVideoList"] as List<string>;
                if (usedVideos == null)
                    args.Context[typePath + "UniqVideoList"] = usedVideos = new List<String>();

                //получаем сохраненный ключ контекста
                string oldKey = args.Context[typePath + "UniqVideoKey"] as string;
                if (oldKey == null)
                    args.Context[typePath + "UniqVideoKey"] = oldKey = "";

                //если ключ поменялся, 
                if (key != oldKey)
                {
                    usedVideos.Clear();
                    args.Context[typePath + "UniqVideoKey"] = key;
                    if (!dbMode)
                    {
                        String file = Path.Combine(GetIniParameter("Main.DataFolder", args), typePath, GetSub(key.Trim()), key);
                        //читаем строки
                        if (File.Exists(file))
                            usedVideos.AddRange(File.ReadAllLines(file).Where(x => x.Length > 0));
                    }
                    else
                    {
                        usedVideos.AddRange(GetDBLines(args, GetIniParameter("Main.DataFolder", args), typePath.Replace(".", "_"), key));
                    }
                }

                if (usedVideos.Count == 0) return "";
                var idx = Rnd.Next(0, usedVideos.Count - 1);
                var line = usedVideos[idx];
                usedVideos.RemoveAt(idx);
                var parts = line.Split('|');
                if (parts.Length < 2) return "";
                var inner = match.Groups[1].Value;
                inner = inner.Replace("[VIDEOCODE]", parts[0]);
                inner = inner.Replace("[VIDEOTITLE]", parts[1]);
                if (parts.Length > 2)
                    inner = inner.Replace("[VIDEODURATION]", parts[2]);
                return inner;
            });
        }

        private string GetKeysFile(PluginMacroArgs args)
        {
            return args.Context["$dbname"] as String ?? "content.db";
        }

        private string[] GetDBLines(PluginMacroArgs args, String dbDir, String table, String key)
        {
            //connect
            var contextKey = "SQLiteConnect";
            SQLiteConnection connection = args.Context[contextKey] as SQLiteConnection;
            if (connection == null)
            {
                connection = new SQLiteConnection("Data Source = \"" + dbDir + "/" + GetKeysFile(args) + "\"; Journal Mode=Off;");
                connection.Open();
                args.Context[contextKey] = connection;
            }

            //reader command
            contextKey = "SQLiteCommand" + table;
            SQLiteCommand reader = args.Context[contextKey] as SQLiteCommand;
            if (reader == null)
            {
                reader = new SQLiteCommand("select content from '" + table + "' where key = :key", connection);
                reader.Parameters.Add(new SQLiteParameter("key", ""));
                args.Context[contextKey] = reader;
                if (!TableExists(table, connection)) reader.CommandText = null;
            }
            if (reader.CommandText == null) return new string[0]; //no table

            reader.Parameters[0].Value = key;
            var content = (reader.ExecuteScalar() as string) ?? "";
            return content.Split(new[] { '\n' }, StringSplitOptions.RemoveEmptyEntries);
        }

        private bool TableExists(String table, SQLiteConnection connection)
        {
            return new SQLiteCommand("SELECT count(*) FROM sqlite_master WHERE name='" + table + "'", connection).ExecuteScalar().ToString() == "1";
        }

        private string GetIniParameter(String name, PluginMacroArgs args)
        {
            String root = args.Context[name] as String;
            if (root == null)
            {
                root = File.ReadAllLines("ContentBox.ini")
                    .FirstOrDefault(x => x.StartsWith(name + "=", StringComparison.InvariantCultureIgnoreCase));
                root = root != null ? root.Split('=')[1] : "";
                args.Context[name] = root;
            }
            return root;
        }

        private string GetSub(string key)
        {
            return key.Length <= 2 ? key : key.Substring(0, 2).Trim();
        }

        private string GetKey(PluginMacroArgs args, Match match)
        {
            int keyIndex;
            string pos = null;
            var result = args.Key;
            if (args.KeyParts != null && (pos = match.Groups["mp"].Value) != "" && (keyIndex = int.Parse(pos.Trim('#'))) > 0)
                result = args.KeyParts[keyIndex > args.KeyParts.Length ? 0 : keyIndex - 1];
            return result;
        }

        private string CleanKey(String key)
        {
            String key1 = key;
            key1 = String.Join("", key1.Split(Path.GetInvalidPathChars()));
            key1 = String.Join("", key1.Split(Path.GetInvalidFileNameChars()));
            return key1;
        }

        public ushort Level { get { return 5; } }
        public String Name { get { return "CBVideoUniqPlugin"; } }
    }
}